==> nui_core-0.0.13.tar.gz: lib/bloc/nui_bloc.dart <==
import 'dart:async';
import 'package:flutter/cupertino.dart';

abstract class NUIBloc<T> {
  StreamController<T> controller = StreamController();
  Stream<T> get stream => controller.stream;
  Sink<T> get sink => controller.sink;
  bool _isDisposed = false;

  void notify(T data){
    if(_isDisposed) return;
    sink.add(data);
  }

  void dispose() {
    _isDisposed = true;
    controller.close();
  }
}

enum BlocStatus{
  LOADING,
  FAILED,
  COMPLETED
}

typedef BlocStreamInitialData<T> = T Function(BuildContext context);
typedef BlocStreamFutureData<T> = Widget Function(BuildContext context, BlocStatus status, T snapshot, Object error);

class NUIBlocStream<T> extends StatefulWidget {
  NUIBlocStream({@required this.bloc, @required this.streamData, @required this.initialData});
  final NUIBloc<T> bloc;
  final BlocStreamFutureData<T> streamData;
  final BlocStreamInitialData<T> initialData;
  @override
  _NUIBlocStreamState<T> createState() => _NUIBlocStreamState<T>();
}

class _NUIBlocStreamState<T> extends State<NUIBlocStream> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
      initialData: widget.initialData(context),
      stream: widget.bloc.stream,
      builder: (BuildContext context, AsyncSnapshot<T> snapshot) {
        if(snapshot.hasData){
          return widget.streamData(context, BlocStatus.COMPLETED, snapshot, null);
        }
        if(snapshot.hasError){
          return widget.streamData(context, BlocStatus.FAILED, null, snapshot.error);
        }
        else{
          return widget.streamData(context, BlocStatus.LOADING, null, null);
        }
      },
    );
  }
}

class NUIBlocWidget<T extends NUIBloc> extends StatefulWidget {
  final Widget child;
  final T bloc;

  const NUIBlocWidget({Key key, @required this.bloc, @required this.child}) : super(key: key);

  @override
  State createState() => _NUIBlocWidgetState();
}

class _NUIBlocWidgetState extends State<NUIBlocWidget> {
  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void dispose() {
    widget.bloc.dispose();
    super.dispose();
  }
}

==> nui_core-0.0.29.tar.gz: lib/bloc/nui_bloc.dart <==
import 'dart:async';
import 'package:flutter/cupertino.dart';

abstract class NUIBloc<T> {
  StreamController<T> controller = StreamController();
  Stream<T> get stream => controller.stream;
  Sink<T> get sink => controller.sink;
  bool _isDisposed = false;

  void notify(T data){
    if(_isDisposed) return;
    sink.add(data);
  }

  void dispose() {
    _isDisposed = true;
    controller.close();
  }
}

enum BlocStatus{
  LOADING,
  FAILED,
  COMPLETED
}

typedef BlocStreamInitialData<T> = T Function(BuildContext context);
typedef BlocStreamFutureData<T> = Widget Function(BuildContext context, BlocStatus status, T snapshot, Object error);

class NUIBlocStreamUI<T> extends StatefulWidget {
  NUIBlocStreamUI({@required this.bloc, @required this.streamData, @required this.initialData});
  final NUIBloc<T> bloc;
  final BlocStreamFutureData<T> streamData;
  final BlocStreamInitialData<T> initialData;
  @override
  _NUIBlocStreamUIState<T> createState() => _NUIBlocStreamUIState<T>();
}

class _NUIBlocStreamUIState<T> extends State<NUIBlocStreamUI> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
      initialData: widget.initialData(context),
      stream: widget.bloc.stream,
      builder: (BuildContext context, AsyncSnapshot<T> snapshot) {
        if(snapshot.hasData){
          return widget.streamData(context, BlocStatus.COMPLETED, snapshot, null);
        }
        if(snapshot.hasError){
          return widget.streamData(context, BlocStatus.FAILED, null, snapshot.error);
        }
        else{
          return widget.streamData(context, BlocStatus.LOADING, null, null);
        }
      },
    );
  }
}

class NUIBlocWidget<T extends NUIBloc> extends StatefulWidget {
  final Widget child;
  final T bloc;

  const NUIBlocWidget({Key key, @required this.bloc, @required this.child}) : super(key: key);

  @override
  State createState() => _NUIBlocWidgetState();
}

class _NUIBlocWidgetState extends State<NUIBlocWidget> {
  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void dispose() {
    widget.bloc.dispose();
    super.dispose();
  }
}
